==> pyTag_0.0.1.tgz: pyTag.py <==
# -*- coding: utf-8 -*-

class pyTag:
    def testTestCase(self):
        return "maanu"

    def __init__(self,filepath):
        self.musicfile = open(filepath,"r+")
        type = self.detectType()
        if type == "ID3" :
            self.tag = id3Tag(self.musicfile)
        else :
            self.tag = None
            self.musicfile.close()

    def Tag(self):
        return self.tag

    def detectType(self):
        tagType = self.musicfile.read(3)
        return tagType

    def closeFile(self):
        self.musicfile.close()

class id3Tag:
    def __init__(self,fd):
        self.mp3file = fd
        self.tagSize = self.getTagSize()
        # 헤더의 크기가 10byte 이므로 0x0a를 더하면 전체 테그의 영역이 나온다.
        self.tagEnd = self.tagSize + 0x0a
        self.getAllTag()

    def getTagSize(self):
        header = self.mp3file.read(7)
        tagSize = (ord(header[3])<<24)+(ord(header[4])<<16)+(ord(header[5])<<8) + ord(header[6])
        return tagSize

    def parseHeader(self,header):
        if header == '\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00' :
            return None
        # 앞에서 4byte 가 ID
        id = header[:4]
        # 다음 4byte 가 프레임 사이즈
        size = (ord(header[4])<<24)+(ord(header[5])<<16)+(ord(header[6])<<8)+ord(header[7])

        # 인코더에 따라서 MCDI 의 사이즈를 아주 좆같이 설정하는 경우가 있다.
        # ID3v2.4 표준에 MCDI 크기의 최대 값은 804byte 이다.
        if id == "MCDI" :
            if size > 804 :
                size = 804

        return {"ID":id,"size":size}

        
    def makeSizeArray(self,size):
        l = [0,0,0,0]
        l[0] = (size & 0xff000000) >> 24
        l[1] = (size & 0xff0000) >> 16
        l[2] = (size & 0xff00) >> 8
        l[3] = (size & 0xff) 

        result = ''
        for i in l:
            result += chr(i)
        return result

    def makeFrameArray(self):
        result = ''
        frameSize = ''
        for f in self.allTags.keys():
            frameSize = self.makeSizeArray(self.allTags[f][0])
            result += (f + frameSize + '\x00\x00' + self.allTags[f][1])
        return result

    def makeTagString(self,str):
        result = [0,'']
        # UTF-8 의 EDB 는 0x03 이다.
        setStr = '\x03' + str + '\x00'
        result[0] = len(setStr)
        result[1] = setStr
        return result

        

    def submitTag(self):
        tag = ''
        # 테그 헤더 : 식별자,버전,플래그
        tagHeader = "ID3\x04\x00\x00"   
        # 테그 헤더 : 사이즈
        tagSize = self.makeSizeArray(self.tagSize)        
        # 테그를 만든다.
        tag = tagHeader + tagSize
        tag += self.makeFrameArray()

        # 파일에 쓴다.        
        self.mp3file.seek(0)
        # 저장 영역을 청소한다.
        for i in range(self.realTagEnd):
            self.mp3file.write('\x00')
        # 다시 맨앞으로 간다음 쓴다.    
        self.mp3file.seek(0)
        self.mp3file.write(tag)

    def setTag(self,id,str):
        if self.allTags.has_key(id) == False :
            return None
        self.allTags[id] = self.makeTagString(str)
        return self.allTags

    def getAllTag(self):
        result = {}
        while(1):
            # 프레임 헤더는 10byte
            frameHeader = self.mp3file.read(10)
            headerInfo = self.parseHeader(frameHeader)
            if headerInfo == None :
                break
            # 헤더의 사이즈 만큼 읽는다. 이만큼이 text 부분
            headerText = self.mp3file.read(headerInfo["size"])
            result[headerInfo["ID"]] = [headerInfo["size"],headerText]

        # Txxx 가 아니면 모두 없앤다.
        for i in result.keys():
            if i[0] != 'T' :
                result.pop(i)

        # 실제 테그가 끝나는 부분
        self.realTagEnd = self.mp3file.tell()
        self.allTags = result
        return result          

    def getTag(self,id):
        if self.allTags.has_key(id) == False :
            return None
        result = self.allTags[id][1][1:]
        return result

    def getAllTagId(self):
        return tuple(self.allTags.keys())

    def detectEDB(self,id):
        if self.allTags.has_key(id) == False :
            return None        
        return self.allTags[id][1][0]

    def addTag(self,id,text):
        if self.allTags.has_key(id) == True :
            return None
        self.allTags[id] = self.makeTagString(text)

    def deleteTag(self,id):
        if self.allTags.has_key(id) == False :
            return None        
        self.allTags.pop(id)

==> pyTag_0.0.2.tgz: pyTag.py <==
# -*- coding: utf-8 -*-

TAGS = ('TALB','TBPM','TCOM','TCON','TCOP','TDAT','TDEN','TDLY','TDOR','TDRC','TDRL','TDTG','TENC','TEXT','TFLT','TIPL','TIME','TIT1','TIT2','TIT3','TKEY','TLAN','TLEN','TMCL','TMED','TMOO','TOAL','TOFN','TOLY','TOPE','TOWN','TPE1','TPE2','TPE3','TPE4','TPOS','TPRO','TPUB','TRCK','TRDA','TRSN','TRSO','TSIZ','TSOA','TSOP','TSOT','TSRC','TSSE','TSST','TXXX','TYER')

class pyTag:
    def testTestCase(self):
        return "maanu"

    def __init__(self,filepath):
        self.musicfile = open(filepath,"r+")
        type = self.detectType()
        if type == "ID3" :
            self.musicfile.close()
            self.tag = id3Tag(filepath)
        else :
            self.tag = None
            self.musicfile.close()

    def Tag(self):
        return self.tag

    def detectType(self):
        tagType = self.musicfile.read(3)
        return tagType

class id3Tag:
    def __init__(self,filepath):
        self.mp3fileName = filepath
        self.mp3file = open(filepath,"r+")

        self.tagSize = self.getTagSize()
        # 헤더의 크기가 10byte 이므로 0x0a를 더하면 전체 테그의 영역이 나온다.
        self.tagEnd = self.tagSize + 0x0a
        self.getAllTag()
        
        # 파일을 닫는다.
        self.mp3file.close()

    def getTagSize(self):
        header = self.mp3file.read(10)
        tagSize = (ord(header[6])<<24)+(ord(header[7])<<16)+(ord(header[8])<<8) + ord(header[9])
        return tagSize

    def parseHeader(self,header):
        if header == '\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00' :
            return None, True 

        ###########################################Its Debug
        #print len(header)
        # 앞에서 4byte 가 ID
        id = header[:4]
        isRight = True
        if TAGS.__contains__(id) == False :
            # 방금 읽은 ID 가 표준에 없는 거라면,
            isRight = False

        # 다음 4byte 가 프레임 사이즈
        size = (ord(header[4])<<24)+(ord(header[5])<<16)+(ord(header[6])<<8)+ord(header[7])


        return {"ID":id,"size":size},isRight

        
    def makeSizeArray(self,size):
        l = [0,0,0,0]
        l[0] = (size & 0xff000000) >> 24
        l[1] = (size & 0xff0000) >> 16
        l[2] = (size & 0xff00) >> 8
        l[3] = (size & 0xff) 

        result = ''
        for i in l:
            result += chr(i)
        return result

    def makeFrameArray(self):
        result = ''
        frameSize = ''
        for f in self.allTags.keys():
            frameSize = self.makeSizeArray(self.allTags[f][0])
            result += (f + frameSize + '\x00\x00' + self.allTags[f][1])
        return result

    def makeTagString(self,str):
        result = [0,'']
        # UTF-8 의 EDB 는 0x03 이다.
        setStr = '\x03' + str + '\x00'
        result[0] = len(setStr)
        result[1] = setStr
        return result

        

    def submitTag(self):
        tag = ''
        # 테그 헤더 : 식별자,버전,플래그
        tagHeader = "ID3\x04\x00\x00"   
        # 테그 헤더 : 사이즈
        tagSize = self.makeSizeArray(self.tagSize)        
        # 테그를 만든다.
        tag = tagHeader + tagSize
        tag += self.makeFrameArray()

        self.mp3file = open(self.mp3fileName,"r+")
       
       # 파일에 쓴다.        
        self.mp3file.seek(0)
        # 저장 영역을 청소한다.
        for i in range(self.realTagEnd):
            self.mp3file.write('\x00')
        # 다시 맨앞으로 간다음 쓴다.    
        self.mp3file.seek(0)
        self.mp3file.write(tag)

        self.mp3file.close()

    def setTag(self,id,str):
        if self.allTags.has_key(id) == False :
            return None
        self.allTags[id] = self.makeTagString(str)
        return self.allTags

    def getAllTag(self):
        # 파일의 맨 앞으로 간다음
        self.mp3file.seek(0)
        # 전체 테그를 한번에 읽는다.
        self.allTagData = self.mp3file.read(self.tagEnd)

        result = {}
        offset = 10;
        while(offset < self.tagEnd):
            # 남은 데이터가 10byte 미만이면 끝낸다.
            if (len(self.allTagData) - offset) < 10 :
                break
            # 프레임 헤더는 10byte
            frameHeader = self.allTagData[offset:offset+10]
            headerInfo,isRight = self.parseHeader(frameHeader)
            if isRight == False :
                offset += 1
                continue
            if headerInfo == None :
                break
            #####################################Its Debug
            #print headerInfo["ID"]
            offset += 10
            # 헤더의 사이즈 만큼 읽는다. 이만큼이 text 부분
            headerText = self.allTagData[offset:offset+headerInfo["size"]]
            ttt = offset
            offset += headerInfo["size"]
            result[headerInfo["ID"]] = [headerInfo["size"],headerText]
            ######################################Its Debug
            #print headerInfo['ID'], hex(ttt), hex(headerInfo['size']), hex(offset)

        # 실제 테그가 끝나는 부분
        self.realTagEnd = offset
        self.allTags = result
        return result          

    def getTag(self,id):
        if self.allTags.has_key(id) == False :
            return None
        result = self.allTags[id][1][1:]
        return result

    def getAllTagId(self):
        return tuple(self.allTags.keys())

    def detectEDB(self,id):
        if self.allTags.has_key(id) == False :
            return None        
        return self.allTags[id][1][0]

    def addTag(self,id,text):
        if self.allTags.has_key(id) == True :
            return None
        self.allTags[id] = self.makeTagString(text)

    def deleteTag(self,id):
        if self.allTags.has_key(id) == False :
            return None        
        self.allTags.pop(id)
